
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import game.*;

public final class Canon extends Sprite implements KeyListener
{
	public static final int CLASS_ID = "Canon".hashCode();	
	private boolean firing = false;
	private int fireCountdown = 0;
	private static int FIRE_INTERVAL = 10;
	private boolean movingEast = false;
	private boolean movingWest = false;
	private boolean fire = false;
	private int bonusType;
	private int bonusCountdown = 0;
	private boolean bonusActive = false;
	private Bonus bonus;

	public Canon(  SpriteVector spriteVector, Component component, int xPosition, int yPosition )
	{
		super( CLASS_ID, spriteVector, component, new Rectangle(xPosition, yPosition, 30, 30), 0 );
		collisionWidth = 70;
		collisionHeight = 60;
		defaultSequence.addImage( "canon.gif" );
		component.addKeyListener(this);
	}

	protected void collisionDetected( Sprite sprite )
	{
		if( sprite.getTypeID() == Missile.CLASS_ID && ((Missile)sprite).getOwner().getTypeID() == CLASS_ID )
		{
			return;
		}
		if( bonusActive && bonusType == Bonus.INVULNERABILITY )
		{
			return;
		}
		else if( sprite.getTypeID() == Monster.CLASS_ID )
		{
			GameProperties.setProperty(Gatesians.DEATH_BY_BORG,1);
			sprite.bury();
		}
		else if( sprite.getTypeID() == Missile.CLASS_ID && ((Missile)sprite).getOwner().getTypeID() != CLASS_ID )
		{
			GameProperties.setProperty(Gatesians.DEATH_BY_MISSILE,1);
			sprite.bury();
		}

	}
	
	public void bonus( Bonus bonus )
	{
		this.bonus = bonus;
		bonusActive = true;
		bonusType = bonus.getType();
		bonusCountdown = 200;
	}

	public void keyPressed( KeyEvent e )
	{
		switch( e.getKeyCode() )
		{
			case KeyEvent.VK_LEFT:
			{
				movingWest = true;
				break;
			}
			case KeyEvent.VK_RIGHT:
			{
				movingEast = true;
				break;
			}
			case KeyEvent.VK_CONTROL:
			{
				firing = true;
				fire = true;
				break;
			}
		}
	}

	public void keyTyped( KeyEvent e ){}
	public void keyReleased( KeyEvent e )
	{
		switch( e.getKeyCode() )
		{
			case KeyEvent.VK_LEFT:
			{
				movingWest = false;
				break;
			}
			case KeyEvent.VK_RIGHT:
			{
				movingEast = false;
				break;
			}
			case KeyEvent.VK_CONTROL:
			{
				firing = false;
				break;
			}
		}
	}

	private int getMissileType()
	{
		if( bonusCountdown % 10 == 0 )
		{
			GameProperties.setProperty( Gatesians.BONUS_LEFT,bonusCountdown);
		}
		if( bonusCountdown <= 0 )
		{
			return Missile.NORMAL;
		}
		switch( bonusType )
		{
			case Bonus.PENETRATING:
			{
				return Missile.PENETRATING;
			}
		}
		return Missile.NORMAL;
	}

	public void draw( Graphics g )
	{	
		int missileType = getMissileType();
		if( (firing || fire) && fireCountdown <= 0 )
	 	{
			fire = false;
			new Missile( this, spriteVector, component,xPosition+getWidth()/2,yPosition,Direction.NORTH, 15, missileType );
			fireCountdown = FIRE_INTERVAL;
		}
		if( movingWest )
		{
			if( xPosition > 10 )
			{
				moveWest(10);
			}
		}
		else if( movingEast )
		{																 
			if( xPosition < component.getBounds().width - getWidth() -10  )
			{
				moveEast(10);
			}
		}
		super.draw( g );
		fireCountdown--;
		if( bonusActive )
		{
			if( bonusType == Bonus.INVULNERABILITY )
			{
				g.setColor(	getInvulnerableColor() );
				g.drawRect( xPosition, yPosition, getWidth(), getHeight() );
				g.drawRect( xPosition+1, yPosition+1, getWidth()-2, getHeight()-2 );
			}
			if( --bonusCountdown == 0 )
			{
				bonus.bury();
				bonusActive = false;
			}
		}
										 
	}

	private Color getInvulnerableColor()
	{
		if( bonusCountdown % 2 == 0 )
		{
			return Color.yellow;
		}
		return Color.red;
	}

}


