
import game.*;
import java.awt.*;
import java.util.*;
import java.awt.event.*;

public class Gatesians implements Runnable, Game
{
	public static final String DEATH_BY_MISSILE = "A";
	public static final String MONSTERS_LEFT = "B";
	public static final String LIVES_LEFT = "C";
	public static final String LEVEL = "D";
	public static final String SCORE = "E";
	public static final String DEATH_BY_BORG = "F";
	public static final String BONUS = "G";
	public static final String BONUS_LEFT = "H";

	private Message message = null;
	private Canon canon;
	private boolean threadSuspended = false;
	private SpriteVector sprites = new SpriteVector();
	private Component component;
	private Graphics graphics;
	private Image offscreenImage;
	private Graphics offscreenGraphics;
	private volatile Thread thread;
	private GameState currentState = null;
	private GameState formerState = null;
	private GameApplet applet;

	public Gatesians( GameApplet applet, Component component )
	{
		GameProperties.game = this;
		this.component = component;
		this.graphics = component.getGraphics();
		this.applet = applet;
		offscreenImage = component.createImage(component.getBounds().width,component.getBounds().height);
		offscreenGraphics = offscreenImage.getGraphics();
		sprites.setBackground( new Background(sprites,component) );
		currentState = new Welcome();
		thread = new Thread(this);
		thread.start();
	}

	public void propertyChanged( String property, Object newValue )
	{
		if( property.equals( BONUS ) )
		{
			message = new Message( this, ((Bonus)newValue).getText() );
			canon.bonus( (Bonus)newValue  );
		}
		else if( property.equals( BONUS_LEFT ) )
		{
			applet.setBonus( ((Integer)newValue).intValue() );
		}
		else if( property.equals( DEATH_BY_MISSILE ) )
		{
			GameProperties.setProperty( LIVES_LEFT, GameProperties.getIntProperty(LIVES_LEFT) -1  );
			nextState( new Death("You were hit.") );
		}
		else if( property.equals( DEATH_BY_BORG ) )
		{
			GameProperties.setProperty( LIVES_LEFT, 0  );
			nextState( new Death("Do not touch the Borg!") );
		}
		else if( property.equals( SCORE ) )
		{
			applet.setScore( ((Integer)newValue).intValue() );
		}
		else if( property.equals( LIVES_LEFT ) )
		{
			applet.setLivesLeft( GameProperties.getIntProperty(property) );
			if( GameProperties.getIntProperty(property) == 0 )
			{
				nextState( new GameOver() );
			}
		}
		else if( property.equals( MONSTERS_LEFT ) )
		{
			if( GameProperties.getIntProperty(LEVEL) == 21 )
			{
				nextState( new TheEnd() );
				return;
			}
			nextState( new NextLevel() );
		}
	}


	private void initLevel()
	{
		GameProperties.setProperty( BONUS_LEFT, 0 );
		sprites.removeAll();
		Monster.init();
		canon = new Canon(sprites,component,200,component.getBounds().height-30);
		for( int i = 0; i < 6; i++ )
		{
			new Monster(sprites,component,i,0);
			new Monster(sprites,component,i,1);
			new Monster(sprites,component,i,2);
			new Monster(sprites,component,i,3);
		}
	}
	

	
	public void start()
	{
		if( thread == null )
		{
			thread = new Thread(this);
			thread.start();
		}
	}

	public void startGame()
	{
		initLevel();
		GameProperties.setProperty( LIVES_LEFT, 3 );
		GameProperties.setProperty( SCORE, 0 );
		GameProperties.setProperty( LEVEL, 0 );
		nextState( new NextLevel() );
		component.requestFocus();
	}

	private void nextState( GameState s )
	{
		formerState = currentState;
		currentState = s;
	}

	public void stopGame()
	{
		nextState( new Welcome() );
		currentState.update();
	}

	public void stop()
	{
		thread = null;
	}


	public void run()
	{
		while(Thread.currentThread() == thread )
		{
			try
			{
				Thread.sleep(30);
				synchronized(this) 
				{
            	while(threadSuspended)
					{
            		wait();
            	}
				}
			}
			catch( Exception e )
			{
			}
			if( currentState != null )
			{
				currentState.update();
			}
			if( message != null )
			{
				message.update();
			}
			graphics.drawImage( offscreenImage, 0,0, component);
		}
	}
	
	public void centerDraw( Graphics g, String s )
	{
		FontMetrics fm = g.getFontMetrics();
		g.drawString( s, (component.getBounds().width-fm.stringWidth(s))/2, (component.getBounds().height-fm.getHeight())/2);
	}

	
	/****************************************************
	*****************************************************
	*** 	AREA	:	INNER CLASSES
	*****************************************************
	*****************************************************/

	private class Message
	{
		private String message;
		private int count = 20;
		private Gatesians gatesians;
		private Font font = new Font("Helvetica",Font.BOLD,16);
		public Message( Gatesians gatesians, String message )
		{
			this.message = message;
			this.gatesians = gatesians;
		}
		
		public void update( )
		{
			if( --count == 0 )
			{
				gatesians.message = null;	
			}
			offscreenGraphics.setColor( Color.white );
			offscreenGraphics.setFont( font );
			centerDraw( offscreenGraphics, message );
		}
	}


	private abstract class GameState
	{
		public abstract void update();//called repeatedly by the game thread
	}


	private class GameRunning extends GameState
	{
		private int bonusCountdown = URandom.getRandomInt(10,500);
		public void update()
		{
			if( bonusCountdown-- == 0 )
			{
				bonusCountdown = URandom.getRandomInt(10,500);
				new Bonus( sprites,component,-10, URandom.getRandomInt(0,200) ,URandom.getRandomInt(0,2) );
			}
			sprites.update( offscreenGraphics );
			Monster.update();
		}
	}

	private class Death extends GameState
	{
		private int time = 30;
		private final Font font = new Font("Helvetica",Font.BOLD,18);
		private String message;
		
		public Death( String message )
		{
			this.message = message;
			sprites.removeAll( Missile.CLASS_ID );
			Monster.stopAllAttacks();
		}

		public void update()
		{
			offscreenGraphics.setColor(Color.yellow);
			offscreenGraphics.setFont(font);
			centerDraw( offscreenGraphics, message + "  Lives left: " + GameProperties.getProperty(LIVES_LEFT) );
			if( --time == 0 )
			{
				nextState(formerState);
			}
		}
	}



	private class GameOver extends GameState
	{
		private final Font font = new Font("Helvetica",Font.BOLD,18);
		private int time = 90;
		
		public void update()
		{
			if( --time == 0 )
			{
				nextState( new Welcome() );
				return;
			}
			offscreenGraphics.setColor( Color.black );
			offscreenGraphics.fillRect( 0, 0, component.getBounds().width, component.getBounds().height );
			offscreenGraphics.setColor(Color.yellow);
			offscreenGraphics.setFont(font);
			centerDraw( offscreenGraphics, "The Borg have assimilated you. Game Over!" );
		}
	}
	
	private class TheEnd extends GameState
	{
		private final Font font = new Font("Helvetica",Font.BOLD,15);
		private int time = 90;
		
		public void update()
		{
			if( --time == 0 )
			{
				nextState( new Welcome() );
				return;
			}
			offscreenGraphics.setColor( Color.black );
			offscreenGraphics.fillRect( 0, 0, component.getBounds().width, component.getBounds().height );
			offscreenGraphics.setColor(Color.yellow);
			offscreenGraphics.setFont(font);
			centerDraw( offscreenGraphics, "Congratulations! The Borg will be back for revenge soon.." );
		}
	}
	
	private class NextLevel extends GameState
	{
		private Font font = new Font("Helvetica",Font.BOLD,18);
		private int time = 30;
		private Color color;
		public NextLevel()
		{
			color = Color.lightGray;
			GameProperties.setProperty(LEVEL,GameProperties.getIntProperty(LEVEL)+1);
			Monster.incrementLevel();
			applet.setLevel(GameProperties.getIntProperty(LEVEL));
		}
		
		public void update()
		{
			if( --time == 0 )
			{
				initLevel();
				nextState( new GameRunning() );
				return;
			}
			offscreenGraphics.setColor(Color.black);
			offscreenGraphics.fillRect(0,0,component.getBounds().width,component.getBounds().height);
			offscreenGraphics.setColor(color);
			offscreenGraphics.setFont(font);
			centerDraw( offscreenGraphics, "Level " + GameProperties.getProperty(LEVEL) );
		}
	}
	
	private class Welcome extends GameState
	{
	
		private Font font = new Font("Helvetica",Font.BOLD,18);
		private Image i = ImageLoader.getImage("borg.jpg");
		private String s1 = "RESISTANCE IS FUTILE";
		private String s2 = "YOU WILL BE ASSIMILATED";
		
		public Welcome()
		{}

		public void update()
		{
			offscreenGraphics.setColor( Color.black );
			offscreenGraphics.fillRect( 0, 0, component.getBounds().width, component.getBounds().height );
			offscreenGraphics.setFont( font );
			offscreenGraphics.setColor( Color.gray );
			FontMetrics fm = offscreenGraphics.getFontMetrics();offscreenGraphics.drawString( s1, (component.getBounds().width-fm.stringWidth(s1))/2, 70  );
			offscreenGraphics.drawString( s2, (component.getBounds().width-fm.stringWidth(s2))/2, component.getBounds().height - 60  );
			offscreenGraphics.drawImage( i, 130, 80, component.getBounds().width-260, component.getBounds().height-160, component  );
		}
	}


}
