
import java.applet.*;
import java.awt.*;
import game.*;

public final class Missile extends Sprite
{

	public static final int PENETRATING = 0;
	public static final int EXPLODING = 2;
	public static final int NORMAL = 1;
	public static final int CLASS_ID = "Missile".hashCode();	
	private int direction;
	private int velocity;
	private int type;
	private Sprite owner;
	private MissileState currentState;

	public Missile( Sprite owner, SpriteVector spriteVector,  Component component, int xPosition, int yPosition, int direction, int type )
	{
		this( owner, spriteVector, component, xPosition, yPosition, direction, 4, type );
	}


	public Missile( Sprite owner, SpriteVector spriteVector,  Component component, int xPosition, int yPosition, int direction, int velocity, int type )
	{
		super( CLASS_ID, spriteVector, component, new Rectangle(xPosition, yPosition,4,6), 0 );
		this.type = type;
		this.velocity = velocity;
		this.direction = direction;
		this.owner = owner;
		collisionHeight = 150;
		collisionWidth = 120;
	}

	public Sprite getOwner()
	{
		return owner;
	}

	public void collisionDetected( Sprite obj )
	{
		if( type == EXPLODING )
		{
			currentState = new Explosion( this );
		}
		if( type == PENETRATING )
		{
			return;
		}
		if( obj.getTypeID() != owner.getTypeID() )
		{
			buried = true;			
		}
	}

	public void refresh( Graphics g )
	{
		g.setColor( Color.red );
		g.fillRect( xPosition-3, yPosition-4, 4, 6 );
	}

	
	public void draw( Graphics g )
	{
		if( yPosition < 0 || yPosition > component.getBounds().height )
		{
			buried = true;
			return;
		}
		g.setColor( Bonus.getColor(type) );
		g.fillRect( xPosition-3, yPosition-4, 4, 6 );
		switch( direction )
		{
			case Direction.NORTH:
			{
				moveNorth(velocity);
				break;
			}
			case Direction.SOUTH:
			{
				moveSouth(velocity);
				break;
			}
			case Direction.WEST:
			{
				moveWest(velocity);
				break;
			}
			case Direction.EAST:
			{
				moveEast(velocity);
				break;
			}
		}
	}

	private abstract class MissileState
	{
		public abstract void update( Graphics g );
	}

	private class Explosion extends MissileState
	{
		Missile missile;

		public Explosion( Missile missile )
		{
			this.missile = missile;
		}

		public void update( Graphics g )
		{}

	}

}



