
import java.util.*;
import game.*;
import java.awt.*;

public class Monster extends Sprite
{

	

	/** AREA COMMENT ************************************
	*****************************************************
	***
	***	AREA	:	STATIC
	***
	*****************************************************
	*****************************************************/

	public static void incrementLevel()
	{
		attackDelayMin -= 1;
		attackDelayMax -= 5;
	}
	
	public final static int CLASS_ID = "Monster".hashCode();
	private static int attackDelayMin = 30;
	private static int attackDelayMax = 150;

	private static boolean paused;
	private static Vector monsters = new Vector();
	private static int count;
	private static boolean east;
	private static int timeToGo;
	private static int timeToAttack;
	private static int eastDisplacement;
	// the Points (column,row) where there is a  spot
	// updated upon attacking and collision with canon missile
	private static Vector freeIndexVector;

	public static int count()
	{
		return monsters.size();
	}
	
	public static void init()
	{
		paused = false;
		monsters = new Vector();
		freeIndexVector = new Vector();
		eastDisplacement = 0;
		count = 0;
		timeToGo = 25;
		timeToAttack = URandom.getRandomInt(attackDelayMin,attackDelayMax);
		east = true;
	}


	public static void update()
	{
		if( --timeToAttack == 0 )
		{	
			for( int i = 0; i < URandom.getRandomInt(1,GameProperties.getIntProperty(Gatesians.LEVEL)+1); i++ )
			{
				Monster m = ((Monster)monsters.elementAt(URandom.getRandomInt(monsters.size())));
				m.attack();
				freeIndexVector.addElement( m.getIndex() );
				timeToAttack = URandom.getRandomInt(attackDelayMin,attackDelayMax);
			}
		}
		if( --timeToGo != 0 )
		{
			return;
		}
		timeToGo = 25;
		if( ++count < 5 )
		{
			if( east )
			{
				moveAll( Direction.EAST, 8 );
			}
			else
			{
				moveAll( Direction.WEST, 8 );
			}									 
		}
		else
		{
			east = !east;
			count = 0;
		}
	}

	public static void stopAllAttacks()
	{
		for( int i = monsters.size()-1; i > -1; i-- )
		{
			((Monster)monsters.elementAt(i)).stopAttack();
		}
	}

	
	private static void pause()
	{
		paused = !paused;
	}

	public static void moveAll( int direction, int x )
	{
		if( paused )
		{
			return;
		}
		if( direction == Direction.EAST )
		{
			eastDisplacement += x;
		}
		else if( direction == Direction.WEST )
		{
			eastDisplacement -= x;
		}
		Monster m;
		for( int i = 0; i < monsters.size(); i++ )
		{
			m = (Monster)monsters.elementAt(i);
			if( m.isAttacking() )
			{
				continue;
			}
			switch( direction )
			{
				case Direction.EAST:
				{
					m.moveEast( x );
					break;
				}
				case Direction.WEST:
				{
					m.moveWest( x );
					break;
				}
				case Direction.NORTH:
				{
					m.moveNorth( x );
					break;
				}
				case Direction.SOUTH:
				{
					m.moveSouth( x );
					break;
				}
			}
		}
	}


	private static int getX( int columnIndex )
	{
		return 40+60*columnIndex;
	}
	private static int getY( int rowIndex )
	{
		return 30*rowIndex+10;
	}

	/** AREA COMMENT ************************************
	*****************************************************
	***
	***	AREA	:	DYNAMIC
	***
	*****************************************************
	*****************************************************/
	private MonsterState currentState = null;
	private boolean attacking = false;
	private Point index;
	private Sequence deathSequence = new Sequence();
	
	public Monster( SpriteVector spriteVector, Component component, int columnIndex, int rowIndex 	 )
	{
		super( CLASS_ID, spriteVector, component, new Rectangle(getX(columnIndex),getY(rowIndex),30,32), 0 );
		defaultSequence.addImage( "target"+(rowIndex+1)+".gif" );
		deathSequence.addImage( "missile_explode1.gif" );
		deathSequence.addImage( "missile_explode2.gif" );
		collisionWidth = 70;
		collisionHeight = 80;
		monsters.addElement(this);
		index = new Point( columnIndex, rowIndex );
	}

	private void stopAttack()
	{
		if( !attacking )
		{
			return;
		}
		attacking = false;
		currentState = new AttackDone( this, spriteVector, component );
	}

	private void setIndex( Point p )
	{
		index = p;
	}

	private Point getIndex()
	{
		return index;
	}

	public boolean equals( Object o )
	{
		try
		{
			return ((Sprite)o).getID() == getID();
		}
		catch( ClassCastException e )
		{
			return false;
		}
	}

	private void attack()
	{
		if( attacking || dead )
		{
			return;
		}
		attacking = true;
		currentState = new Attack( this, spriteVector, component );
	}

	private void setState( MonsterState s )
	{
		currentState = s;
	}

	private boolean isAttacking()
	{
		return attacking;
	}

	private int getScore()
	{
		if( attacking )
		{
			return 2 * (40 - (10*index.y));
		}
		return 40 - (10*index.y);
	}

	public void collisionDetected( Sprite obj )
	{
		if( obj.getTypeID() == Canon.CLASS_ID || (obj.getTypeID() == Missile.CLASS_ID && ((Missile)obj).getOwner().getTypeID() != CLASS_ID)  )
		{
			currentState = new Death( this, spriteVector, component );
		}
	}


	public void draw( Graphics g )
	{
		super.draw(g);
		if( currentState != null  )
		{
			currentState.update(g);
		}
	}

	/** AREA COMMENT ************************************
	*****************************************************
	***
	*** 	AREA	:	INNER CLASSES
	***
	*****************************************************
	*****************************************************/


	/** INNER CLASS COMMENT *******************************************************
	**
	** 		INNER CLASS	: MonsterState
	**
	**			DESCRIPTION	: 	
	**
	**
	*******************************************************************************/

	private abstract class MonsterState
	{
		protected Monster monster;
		protected SpriteVector spriteVector;
		protected Component component;
		
		public MonsterState( Monster monster, SpriteVector spriteVector, Component component )
		{
			this.monster = monster;	
			this.spriteVector = spriteVector;
			this.component = component;
		}
	
		public abstract void update( java.awt.Graphics g );
	}
	
	/** INNER CLASS COMMENT *******************************************************
	**
	** 		INNER CLASS	: 	AttackDone
	**			DESCRIPTION	: 	Defines the behaviour of the monster when done 
	**								attacking (and survived). The monster attaches itself 
	**								to the "army"
	*******************************************************************************/
	private class AttackDone extends MonsterState
	{
		
		private int targetY;


		public AttackDone( Monster monster, SpriteVector spriteVector, Component component )
		{
			super( monster, spriteVector, component );
			
				Monster.pause();
			// find empty place
				Point empty = findEmpty();
				monster.setIndex( empty );
			// initialize attachment
				targetY = getY(empty.y);
				monster.setXPosition( monster.getX(empty.x)+monster.eastDisplacement );
				monster.setYPosition( 0 );
		}

		private Point findEmpty()
		{
			for( int i = freeIndexVector.size()-1; i > -1; i-- )
			{
				Point p = (Point)freeIndexVector.elementAt( i );
				if( p.y == getIndex().y )
				{
					freeIndexVector.removeElementAt( i );
					return p;
				}
			}
			return null;//should never happen - can it happen??
		}


		public void update( Graphics g )
		{
			int m = 10;
			// when done, restart army movement
			if( targetY == monster.getYPosition() )
			{
				Monster.pause();
				monster.setState( null );
				return;
			}
			else if( targetY - monster.getYPosition() < 10 )
			{
				m = targetY - monster.getYPosition();
			}
			monster.moveSouth(m);
		}

	}
	/** INNER CLASS COMMENT *******************************************************
	**
	** 		INNER CLASS	: Attack
	**
	**			DESCRIPTION	: Defines the behaviour of the monster when attacking
	**
	**
	*******************************************************************************/
	private class Attack extends MonsterState
	{
	
		private int fireCountDown = URandom.getRandomInt(5,20);
		private boolean eastAttack = URandom.getRandomInt(2) == 1;
		private int directionChange = URandom.getRandomInt(10,100);
		private int horisontalVelocity;
		private int verticalVelocity;
	
		public Attack( Monster monster, SpriteVector spriteVector, Component component )
		{
			super( monster, spriteVector, component );
			horisontalVelocity = 8 + ((3-monster.getIndex().y)*2);
			verticalVelocity = 3 + (3-monster.getIndex().y);
		}
	
	
		public void update( Graphics g )
		{
			int xPosition = monster.getXPosition();
			int yPosition = monster.getYPosition();
			if( yPosition >= component.getBounds().height )
			{
				monster.stopAttack();
				return;
			}
			if( fireCountDown-- == 0 )
			{
				new Missile(monster,spriteVector,component,xPosition,yPosition+30,Direction.SOUTH,Missile.NORMAL);
				fireCountDown = URandom.getRandomInt( 5, 30 );
			}
			if( xPosition > component.getBounds().width )
			{
				eastAttack = false;
				directionChange = URandom.getRandomInt(30,100);
			}
			else if( xPosition < -monster.getWidth() )
			{
				eastAttack = true;
				directionChange = URandom.getRandomInt(30,100);
			}
			else if( --directionChange == 0 )
			{
				directionChange = URandom.getRandomInt(30,100);
				eastAttack = !eastAttack;
			}
			if( eastAttack )
			{
				monster.moveEast(horisontalVelocity);
			}
			else
			{
				monster.moveWest(horisontalVelocity);
			}
			monster.moveSouth(verticalVelocity);
		}
	}
	

	private class Death extends MonsterState
	{

		private Font font = new Font("Helvetica",Font.BOLD,14);
		private int score;

		public Death(Monster monster, SpriteVector spriteVector, Component component )
		{
			super( monster, spriteVector, component );
			score = monster.getScore();
			GameProperties.setProperty( Gatesians.SCORE, GameProperties.getIntProperty( Gatesians.SCORE ) + score );
			monster.setSequence( deathSequence );
			monster.kill();
			monster.attacking = false;
		}

		public void update( Graphics g )
		{
			g.setColor(Color.white);
			g.setFont(font);
			g.drawString( ""+score, monster.getXPosition()+15, monster.getYPosition()+15 );
			if( monster.getSequence().isComplete() )
			{
				freeIndexVector.addElement( index );
				monsters.removeElement(monster);
				if( monsters.size() == 0 )
				{
					GameProperties.setProperty( Gatesians.MONSTERS_LEFT, 0);
				}
				monster.bury();
			}
		}
	}

}
