package game;

import java.awt.*;
import java.awt.image.*;
import java.applet.*;
import java.util.*;

public abstract class Sprite
{
	protected int collisionWidth = 90;//percent
	protected int collisionHeight = 90;//percent

	protected boolean dead = false;
	protected boolean buried = false;
	protected int zOrder;
	protected Component component;
	protected Sequence defaultSequence = new Sequence();
	protected Sequence sequence = defaultSequence;
	protected int xPosition;
	protected int yPosition;
	protected Rectangle collisionRect = new Rectangle(0,0,0,0);
	private int typeID;
	protected SpriteVector spriteVector;
	protected int height;
	protected int width;
	private static int idCount = 0;
	private int id;

	public Sprite( int typeID, SpriteVector spriteVector, Component component, Rectangle bounds, int zOrder )
	{
		this.spriteVector = spriteVector;
		this.typeID = typeID;
		this.xPosition = bounds.x;
		this.yPosition = bounds.y;
		this.width = bounds.width;
		this.height = bounds.height;
		this.zOrder = zOrder;
		this.component = component;
		spriteVector.addSprite(this);
		id = idCount++;
	}

	public void setSequence( Sequence sequence )
	{
		this.sequence = sequence;
	}

	public Sequence getSequence()
	{
		return sequence;
	}

	public int getID()
	{
		return id;
	}

	public void event( EventObject e )
	{}

	public void setXPosition( int xPosition )
	{
		this.xPosition = xPosition;
	}

	public void setYPosition( int yPosition )
	{
		this.yPosition = yPosition;
	}
	
	public int getXPosition()
	{
		return xPosition;
	}

	public int getYPosition()
	{
		return yPosition;
	}

	public int getTypeID()
	{
		return typeID;
	}

	public int getZOrder()
	{
		return zOrder;
	}

	public void moveEast( int n )
	{
		xPosition += n;
	}

	public void moveWest( int n )
	{
		xPosition -= n;
	}
	
	public void moveNorth( int n )
	{
		yPosition -= n;
	}

	public void moveSouth( int n )
	{
		yPosition += n;
	}

	public void kill()
	{
		dead = true;
	}

	public boolean isDead()
	{
		return dead;
	}
	
	public void bury()
	{
		buried = true;
	}

	public boolean isBuried()
	{
		return buried;
	}


	public int getHeight()
	{
		return height;
	}

	public int getWidth()
	{
		return width;
	}

	public Rectangle getCollisionRect()
	{
		collisionRect.width = getWidth()*collisionWidth/100;
		collisionRect.height = getHeight()*collisionHeight/100;
		collisionRect.x = xPosition+getWidth()*(100-collisionWidth)/200;
		collisionRect.y = yPosition+getHeight()*(100-collisionHeight)/200;
		return collisionRect;
	}

	public void checkCollision( Sprite sprite )
	{
		if( sprite == this )
		{
			return;
		}
		if( getCollisionRect().intersects( sprite.getCollisionRect() ) )
		{
			collisionDetected( sprite );
		}
	}

	protected abstract void collisionDetected( Sprite sprite );
	
	public void refresh( Graphics g )
	{
		if( sequence.getImage() != null )
		{
			g.drawImage( sequence.getImage(), xPosition, yPosition, component );
		}
	}

	public void draw( Graphics g )
	{
		g.drawImage( sequence.advance(), xPosition, yPosition, component );
	/*
		Rectangle r = getCollisionRect();
		g.setColor( Color.yellow );
		g.drawRect( r.x, r.y, r.width, r.height );
		g.setColor( Color.red );
		g.drawRect( xPosition, yPosition, getWidth(), getHeight() );
	*/
	}

}
